@@ -24,7 +24,6 @@
  */
 
 #include "test.h"
-__FBSDID("$FreeBSD$");
 
 #define should(__a, __code, __opts) \
 assertEqualInt(__code, archive_read_set_options(__a, __opts))
@@ -116,7 +115,7 @@ test(int pristine)
 		    archive_error_string(a));
 	}
 
-	archive_read_finish(a);
+	archive_read_free(a);
 }
 
 DEFINE_TEST(test_archive_read_set_options)
