@@ -1,22 +1,24 @@
 DEFINE_TEST(test_0)
-DEFINE_TEST(test_basic)
 DEFINE_TEST(test_C)
+DEFINE_TEST(test_I)
+DEFINE_TEST(test_L)
+DEFINE_TEST(test_P)
+DEFINE_TEST(test_Z1)
+DEFINE_TEST(test_basic)
 DEFINE_TEST(test_d_before_zipfile)
 DEFINE_TEST(test_d_after_zipfile)
+DEFINE_TEST(test_doubledash)
 DEFINE_TEST(test_glob)
 DEFINE_TEST(test_j)
-DEFINE_TEST(test_L)
 DEFINE_TEST(test_n)
 DEFINE_TEST(test_not_exist)
 DEFINE_TEST(test_o)
 DEFINE_TEST(test_p)
-DEFINE_TEST(test_P)
 DEFINE_TEST(test_q)
 DEFINE_TEST(test_singlefile)
-DEFINE_TEST(test_t_bad)
 DEFINE_TEST(test_t)
+DEFINE_TEST(test_t_bad)
 DEFINE_TEST(test_version)
 DEFINE_TEST(test_x_single)
 DEFINE_TEST(test_x_multiple)
 DEFINE_TEST(test_x_multiple_with_d)
-DEFINE_TEST(test_Z1)
